@@ -14,7 +14,8 @@ from scipy import linalg as la
 from scipy import stats
 import scipy as sp
 import pylab
-from common import _cost_rec, _cost_cod, _st, _replaceAtoms, _saveDict
+from common import _cost_rec, _cost_cod, _replaceAtoms, _saveDict
+from prox import _st
 ra = sp.random
 
     
@@ -164,7 +165,10 @@ def _ist(X, U0, D, C, pars, maxiter=1000):
     ####### pre-computes what is needed in the loop
     DtD /= (N*d*sigma0)
     DtX = sp.dot(D.T, X) / (N*d*sigma0)
-    CX = sp.dot(C, X)
+    if C != None:
+        CX = sp.dot(C, X)
+    else:
+        CX = sp.zeros(U.shape)
     # constant part of the gradient descent step
     U0 = pars['eta']*CX/(N*K*sigma0) + DtX
     # first-order part of the gradient descent step
@@ -174,6 +178,9 @@ def _ist(X, U0, D, C, pars, maxiter=1000):
     for i in xrange(maxiter):
         # compute gradient step and soft-thresholding
         Unew = _st(U0 - sp.dot(A, Y), pars['tau']/(N*K*sigma0))
+        # non-negativity
+        if pars['nnU']:
+            Unew = sp.clip(Unew, 0, sp.inf)
         # evaluates the cost function
         E_new = _cost_fast(X, Unew, D, CX, pars)
         if i%1 == 0 and pars['verbose']:
@@ -224,7 +231,7 @@ def _pgd_fixed(A0, X, U, B, G2, cost, maxiter=500, pars=None, sigma=None, axis=0
         E = Enew
     return A, j
 
-def _pgd(Y0, ABt, BBt, cost, maxiter=500, axis=0, bound=1, verbose=False, rtol=1.e-4):
+def _pgd(Y0, ABt, BBt, cost, maxiter=500, axis=0, bound=1, verbose=False, rtol=1.e-4, nn=False):
     '''
     Projected gradient descent with FISTA acceleration.
 
@@ -278,6 +285,9 @@ def _pgd(Y0, ABt, BBt, cost, maxiter=500, axis=0, bound=1, verbose=False, rtol=1
     for j in xrange(maxiter):
         ###### gradient descent step
         Ynew = Z + (1/sigma) * (ABt - sp.dot(Z, BBt))
+        ###### non-negativity constraint
+        if nn:
+            Ynew = sp.clip(Ynew, 0, sp.inf)
         ###### projections onto the ball
         if bound > 0:
             # computes the norms
@@ -333,6 +343,10 @@ def learn(X, D0, C0, U0, callable=None, **kwargs):
           Weight of the coding error (default 1.0)
     mu : float, optional
           Weight of the l2 penalty on the coefficients (default 0.0)
+    nnU : bool, optional
+          Adds a non-negativity constraint on U (default False)
+    nnD : bool, optional
+          Adds a non-negativity constraint on U (default False)
     maxiter : int, optional
           Maximum number of outer iterations (default 10)
     minused : integer, optional
@@ -372,6 +386,9 @@ def learn(X, D0, C0, U0, callable=None, **kwargs):
         'tau'    : 0.1,
         'eta'    : 1.0,
         'mu'     : 0.0,
+        'nnU'    : False,
+        'nnD'    : False,
+        'Cbound' : 1.0,
         'verbose': False,
         'rtol'   : 1.e-4,
         'save_dict': False,
@@ -457,7 +474,7 @@ def learn(X, D0, C0, U0, callable=None, **kwargs):
         def _costD(Y):
             return _cost_rec(Y, X, U)
         f = 1./(N*d)
-        D, iters = _pgd(D, f*XUt, f*UUt, _costD, verbose=pars['verbose'], rtol=pars['rtol'])
+        D, iters = _pgd(D, f*XUt, f*UUt, _costD, verbose=pars['verbose'], rtol=pars['rtol'], nn=pars['nnD'])
         timeB = time.time() - start
         # check the value of the cost function
         E, full_out = _cost(X, U, D, C, pars)
@@ -473,7 +490,7 @@ def learn(X, D0, C0, U0, callable=None, **kwargs):
             def _costC(Y):
                 return _cost_cod(Y, X, U, pars)
             f = pars['eta']/(N*K)
-            C, iters = _pgd(C, f*XUt.T, f*XXt, _costC, axis=1, verbose=pars['verbose'], rtol=pars['rtol'])
+            C, iters = _pgd(C, f*XUt.T, f*XXt, _costC, axis=1, bound=pars['Cbound'], verbose=pars['verbose'], rtol=pars['rtol'])
             timeC = time.time() - start
             # check the value of the cost function
             E, full_out = _cost(X, U, D, C, pars)
@@ -503,7 +520,8 @@ def learn(X, D0, C0, U0, callable=None, **kwargs):
     full_out['time'] = timing
     if callable != None:
         full_out['call_res'] = call_res
-
+    full_out['iters'] = i
+    
     return D, C, U, full_out
 
 def init(X, K, det=False, rnd=False):
@@ -576,10 +594,15 @@ if __name__=='__main__':
     d = 20  # dimension of the data
 
     ############ synthetic experiment
+
+    NMF = True
     
     # generating dictionary, all atoms are centered on 0
     # and with 2-norm one
-    D_true = ra.uniform(-1, 1, size=(d, K)).astype(sp.float32)
+    if NMF:
+        D_true = ra.uniform(0, 1, size=(d, K)).astype(sp.float32)
+    else:
+        D_true = ra.uniform(-1, 1, size=(d, K)).astype(sp.float32)
     #D_true -= sp.mean(D_true, 0) 
     D_true /= sp.sqrt(sp.sum(D_true**2, 0)).reshape((1, -1))
     #assert sp.allclose(sp.mean(D_true, 0), 0, atol=1.e-7), sp.mean(D_true, 0)
@@ -587,7 +610,10 @@ if __name__=='__main__':
     # generating coefficients
     U_true = sp.zeros((K, N), sp.float32)
     for i in xrange(N):
-        U_true[ra.permutation(K)[:k],i] = ra.uniform(-1, 1, size=(k,))
+        if NMF:
+            U_true[ra.permutation(K)[:k],i] = ra.uniform(0, 1, size=(k,))
+        else:
+            U_true[ra.permutation(K)[:k],i] = ra.uniform(-1, 1, size=(k,))
     assert sp.all(sp.sum(sp.where(sp.absolute(U_true) > 0, 1, 0), 0) == k)
     #m = sp.mean(U_true, 0)
     #U_true -= m.reshape((1, -1))
@@ -607,15 +633,12 @@ if __name__=='__main__':
 
     D0, C0, U0 = init(X, K)
 
-    pars = {
-        'tau' : .3, # sparsity coefficient
-        'mu'  : 1.e-8, # l2 regularization
-        'eta' : 1, # coding/decoding weight
-        'maxiter' : 80,
-        'minused' : 1,
-        }
-    #dic_reg = 1.
-    D, C, U, full_out = learn(X, D0, C0, U0, tau=0.3, mu=1.e-8, eta=1., maxiter=80, minused=1)
+    #D0 = D_true + ra.normal(0, 0.2, size=(d, K))
+    tau = .1
+    mu = 1.e-8
+    eta = 0.
+    maxiter = 80
+    D, C, U, full_out = learn(X, D0, C0, U0, tau=tau, mu=mu, eta=eta, maxiter=maxiter)
 
     # comparison wih true dictionary
     print
@@ -629,4 +652,14 @@ if __name__=='__main__':
     print dist
     print sp.where(dist < 0.01)
     print len(sp.where(dist < 0.01)[0])
-    
+
+    if NMF:
+        print sp.where(U < 0), sp.where(D < 0)
+        D, C, U, full_out = learn(X, D0, C0, U0, tau=tau, mu=mu, eta=eta, maxiter=maxiter, nnU=True, nnD=True, verbose=False)
+        print sp.where(U < 0), sp.where(D < 0)
+        d2 = sp.sum((D[:,:,sp.newaxis] - D_true[:,sp.newaxis,:])**2, 0)
+        closest = sp.argmin(d2, 0)
+        dist = 1-sp.absolute(sp.sum(D[:,closest] * D_true, 0))
+        print dist
+        print sp.where(dist < 0.01)
+        print len(sp.where(dist < 0.01)[0])
