@@ -9,7 +9,7 @@ Dual Dictionaries LEarning", DISI-TR-2010-XX, 2010.
 #import numpy as NP
 import scipy as sp
 from scipy import linalg as la
-from paddle import dual, ntframes, common
+from paddle import dual, tframes, common
 import pylab, os
 
 def principal_angle(A, B):
@@ -108,7 +108,7 @@ def plot_figure_1(call_res, figfn, Erec_pca):
 def run_experiment_2(d, k, N, K, noise, exp2fn):
 
     # build a normalized tight frame
-    A = ntframes.get(d, K)
+    A = tframes.get(d, K)
     # generate the coefficients of the atoms combinations
     Utrue = sp.zeros((K, N), sp.float32)
     for i in xrange(N):
